from locals import *
from animation import NoAnimation, Animation, AnimationFrame
from level_asset import LevelAsset

class Assets(object):
    @classmethod
    def _fullfn(cls, fn):
        return os.path.join("data", fn)

    @classmethod
    def _frame(cls, fn, *args):
        mirror_rects = fn.endswith('?mirror')
        return AnimationFrame(cls._image(fn), *args, mirror_rects = mirror_rects)

    @classmethod
    def _frames(cls, anim):
        return [cls._frame(*args) for args in anim]

    @classmethod
    def _image(cls, img_fn):
        mirror = img_fn.endswith('?mirror')
        img_fn = img_fn.replace('?mirror', '')

        img = pygame.image.load(cls._fullfn(img_fn)).convert_alpha()

        if mirror:
            img = pygame.transform.flip(img, True, False).convert_alpha()

        return img

    @classmethod
    def _doubled(cls, img_fn, orientation):
        img = cls._image(img_fn)
        rect = img.get_rect()

        # Create 2 copies of the image side by side
        if orientation.endswith('horizontal'):
            double_img = pygame.Surface((rect.width * 2, rect.height), img.get_flags(), img)
            double_img.blit(img, (0, 0))
            double_img.blit(img, (rect.width, 0))
            single_size = rect.width
        elif orientation.endswith('vertical'):
            double_img = pygame.Surface((rect.width, rect.height * 2), img.get_flags(), img)
            double_img.blit(img, (0, 0))
            double_img.blit(img, (0, rect.height))
            single_size = rect.height

        superscrolling = orientation.startswith('super')

        return double_img.convert_alpha(), single_size, superscrolling

    @classmethod
    def _make_font(cls, fn, size):
        return pygame.font.Font(cls._fullfn(fn), size)

    @classmethod
    def _make_sfx(cls, sfx_arg):
        if isinstance(sfx_arg, tuple):
            sfx_fn, sfx_volume = sfx_arg
        else:
            sfx_fn, sfx_volume = sfx_arg, 1.0

        # hack to "increase" voice volume to 200%
        if 'voices' not in sfx_fn:
            sfx_volume *= 0.5

        snd = pygame.mixer.Sound(cls._fullfn(sfx_fn))
        if sfx_volume is not None:
            snd.set_volume(sfx_volume)

        return { 'fn' : sfx_fn, 'sound' : snd } # last moment hack

    def __init__(self, IMAGES = None, BACKGROUNDS = None, ANIMATIONS = None, FONTS = None,
                    SFX = None, LEVELS = None):

        if IMAGES is None:
            IMAGES = {}
        if BACKGROUNDS is None:
            BACKGROUNDS = {}
        if ANIMATIONS is None:
            ANIMATIONS = {}
        if FONTS is None:
            FONTS = {}
        if SFX is None:
            SFX = {}
        if LEVELS is None:
            LEVELS = {}

        self.IMAGES = IMAGES
        self.BACKGROUNDS = BACKGROUNDS
        self.ANIMATIONS = ANIMATIONS
        self.FONTS = FONTS
        self.SFX = SFX
        self.LEVELS = LEVELS

    def load_assets(self):
        images = { img_name : NoAnimation(self._image(img_fn))
                   for img_name, img_fn in self.IMAGES.items() }
        backgrounds = { img_name : self._doubled(*args)
                        for img_name, args in self.BACKGROUNDS.items() }
        animations = { anim_name : Animation(self._frames(anim))
                       for anim_name, anim in self.ANIMATIONS.items() }
        fonts = { font_name : self._make_font(*args) for font_name, args in self.FONTS.items() }
        sfx = { sfx_name : self._make_sfx(args) for sfx_name, args in self.SFX.items() }
        levels = { level_name : LevelAsset(self._fullfn(level_fn))
                   for level_name, level_fn in self.LEVELS.items() }

        assets = {}
        assets.update(images)
        assets.update(backgrounds)
        assets.update(animations)
        assets.update(fonts)
        assets.update(sfx)
        assets.update(levels)

        return assets
