from locals import *
from game import g
from character import Character

class BulletBase(Character):
    WEAPON_SOUNDS = []
    USE_SFX = None
    HIT_SFX = None
    WALK_ACCEL = 0             # pixels per second^2
    WALK_ACCEL_DURATION = 0    # seconds of acceleration

    def __init__(self):
        Character.__init__(self, 0, 0, g.level.player.dir_facing)

        self.can_fall = False

        self.disp_rect.centery = g.level.player.char_rect.centery
        if g.level.player.dir_facing == DIR_RIGHT:
            self.disp_rect.left = g.level.player.char_rect.right
        else:
            self.disp_rect.right = g.level.player.char_rect.left

        self.pos = self.char_rect.midbottom # "does nothing" but needed to update fpos too...
        self.start_walk()

        self.shoot_animation = g.assets[self.SHOOT_ANIMATION[g.level.player.dir_facing]]
        self.shoot_animation.restart_anim(lambda : g.level.player.resume_after_melee_or_shoot())

        self.bullet_death = g.get_ticks() + self.LIFETIME

    def walked_into_wall(self):
        self.hit_something()

    def update(self, *args):
        Character.update(self, *args)

        if g.get_ticks() >= self.bullet_death:
            self.kill()

    def hit_something(self):
        self.kill()

        if self.HIT_SFX is not None:
            g.assets[self.HIT_SFX]['sound'].play()

class Pistol(BulletBase):
    ATTACK_TYPE = ATTACK_PISTOL
    ICON = "icon-pistol"
    SHOOT_ANIMATION = { DIR_LEFT : "dgp-pistol-left", DIR_RIGHT : "dgp-pistol-right" }
    ANIMATION = { DIR_LEFT: "pistol-bullet-left", DIR_RIGHT : "pistol-bullet-right" }

    LIFETIME = 750 # msecs
    INITIAL_WALK_VELOC = 800   # pixels per second

    WEAPON_SOUNDS = [ "pistol1", "pistol2", "pistol3" ]
    USE_SFX = 'gun'

class Bazooka(BulletBase):
    ATTACK_TYPE = ATTACK_BAZOOKA
    ICON = "icon-bazooka"
    SHOOT_ANIMATION = { DIR_LEFT : "dgp-bazooka-left", DIR_RIGHT : "dgp-bazooka-right" }
    ANIMATION = { DIR_LEFT: "bazooka-bullet-left", DIR_RIGHT : "bazooka-bullet-right" }

    LIFETIME = 1000 # msecs
    INITIAL_WALK_VELOC = 600   # pixels per second

    WEAPON_SOUNDS = [ "bazooka1", "bazooka2", "bazooka3" ]
    USE_SFX = 'bazooka-shot'
    HIT_SFX = 'bazooka-hit'

class Mustache(BulletBase):
    ATTACK_TYPE = ATTACK_MUSTACHE
    ICON = "icon-mustache"
    SHOOT_ANIMATION = { DIR_LEFT : "dgp-mustache-left", DIR_RIGHT : "dgp-mustache-right" }
    ANIMATION = { DIR_LEFT: "mustache-bullet-left", DIR_RIGHT : "mustache-bullet-right" }

    LIFETIME = 250 # msecs
    INITIAL_WALK_VELOC = 2000  # pixels per second

    WEAPON_SOUNDS = [ "moustache1", "moustache2", "moustache3" ]
    USE_SFX = 'mustach__woosh'
