from character import Character
from immunities import Immunities
from locals import *
from game import g
import random

####################################################################################################

class Enemy(Character):
    DEATH_SOUND = None

    IMM_OVERLAY_ORDER = [
        ATTACK_BAZOOKA, ATTACK_PISTOL, ATTACK_PUNCH, ATTACK_KICK, ATTACK_BELT, ATTACK_MUSTACHE,
        ATTACK_STOMP, ATTACK_LASER, ATTACK_CLUB, ATTACK_FLAMETHROWER
    ]

    INITIAL_WALK_VELOC = 100   # pixels per second
    WALK_ACCEL = 150           # pixels per second^2
    WALK_ACCEL_DURATION = 0.2  # seconds of acceleration

    def __init__(self, initial_x, initial_y):
        Character.__init__(self, initial_x, initial_y, DIR_LEFT)

        self.immunities = Immunities(self)
        self.prepare_animations()
        self.randomize_values()

    def prepare_animations(self):
        self.walk_right_anim = self.prepare_one_animation(self.walk_right_anim, self.IMMUNITY_ANIMATION[DIR_RIGHT])
        self.walk_left_anim = self.prepare_one_animation(self.walk_left_anim, self.IMMUNITY_ANIMATION[DIR_LEFT])
        self.idle_right_anim = self.prepare_one_animation(self.idle_right_anim, self.IMMUNITY_ANIMATION[DIR_RIGHT])
        self.idle_left_anim = self.prepare_one_animation(self.idle_left_anim, self.IMMUNITY_ANIMATION[DIR_LEFT])
        self.jump_right_anim = self.prepare_one_animation(self.jump_right_anim, self.IMMUNITY_ANIMATION[DIR_RIGHT])
        self.jump_left_anim = self.prepare_one_animation(self.jump_left_anim, self.IMMUNITY_ANIMATION[DIR_LEFT])

    def prepare_one_animation(self, anim, overlays):
        for attack in self.IMM_OVERLAY_ORDER:
            if attack in self.immunities.immunity_list:
                anim = anim.overlay(g.assets[overlays[attack]])

        return anim

    def death_sound(self):
        if self.DEATH_SOUND is not None:
            g.assets[self.DEATH_SOUND]['sound'].play()

    @staticmethod
    def _symmetric_rand(num):
        return int(round(random.uniform(-num, num)))

    def randomize_values(self):
        # Slightly randomize walk speeds
        self.walk_speed_tweak = self._symmetric_rand(20)
        self.walk_accel_tweak = self._symmetric_rand(30)

    def update(self, *args):
        Character.update(self, *args)

        if self.char_rect.top >= SCREEN_HEIGHT:
            g.level.kill_enemy_no_attack(self)

    def change_direction(self):
        # Change direction
        self.stop_walk()

        if self.dir_facing == DIR_LEFT:
            self.start_walk_right()
        else:
            self.start_walk_left()

    def walked_into_wall(self):
        self.change_direction()

####################################################################################################

class GreenEnemy(Enemy):
    DEATH_SOUND = 'Gnorlm-green'

    ANIMATION = { DIR_LEFT : "green-walk-left", DIR_RIGHT : "green-walk-right" }
    IMMUNITY_ANIMATION = {
        DIR_LEFT : {
            ATTACK_STOMP : "green-antistomp-left",
            ATTACK_PUNCH : "green-antipunch-left",
            ATTACK_KICK : "green-antikick-left",
            ATTACK_BELT : "green-antibelt-left",
            ATTACK_CLUB : "green-anticlub-left",
            ATTACK_PISTOL : "green-antipistol-left",
            ATTACK_FLAMETHROWER : "green-antiflamethrower-left",
            ATTACK_BAZOOKA : "green-antibazooka-left",
            ATTACK_LASER : "green-antilaser-left",
            ATTACK_MUSTACHE : "green-antimustache-left"
        },

        DIR_RIGHT : {
            ATTACK_STOMP : "green-antistomp-right",
            ATTACK_PUNCH : "green-antipunch-right",
            ATTACK_KICK : "green-antikick-right",
            ATTACK_BELT : "green-antibelt-right",
            ATTACK_CLUB : "green-anticlub-right",
            ATTACK_PISTOL : "green-antipistol-right",
            ATTACK_FLAMETHROWER : "green-antiflamethrower-right",
            ATTACK_BAZOOKA : "green-antibazooka-right",
            ATTACK_LASER : "green-antilaser-right",
            ATTACK_MUSTACHE : "green-antimustache-right"
        }
    }

    def about_to_fall(self):
        self.change_direction()

class RedEnemy(Enemy):
    DEATH_SOUND = 'Knarxal-brown'

    ANIMATION = { DIR_LEFT : "red-walk-left", DIR_RIGHT : "red-walk-right" }
    IMMUNITY_ANIMATION = {
        DIR_LEFT : {
            ATTACK_STOMP : "red-antistomp-left",
            ATTACK_PUNCH : "red-antipunch-left",
            ATTACK_KICK : "red-antikick-left",
            ATTACK_BELT : "red-antibelt-left",
            ATTACK_CLUB : "red-anticlub-left",
            ATTACK_PISTOL : "red-antipistol-left",
            ATTACK_FLAMETHROWER : "red-antiflamethrower-left",
            ATTACK_BAZOOKA : "red-antibazooka-left",
            ATTACK_LASER : "red-antilaser-left",
            ATTACK_MUSTACHE : "red-antimustache-left"
    },

        DIR_RIGHT : {
            ATTACK_STOMP : "red-antistomp-right",
            ATTACK_PUNCH : "red-antipunch-right",
            ATTACK_KICK : "red-antikick-right",
            ATTACK_BELT : "red-antibelt-right",
            ATTACK_CLUB : "red-anticlub-right",
            ATTACK_PISTOL : "red-antipistol-right",
            ATTACK_FLAMETHROWER : "red-antiflamethrower-right",
            ATTACK_BAZOOKA : "red-antibazooka-right",
            ATTACK_LASER : "red-antilaser-right",
            ATTACK_MUSTACHE : "red-antimustache-right"
        }
    }

####################################################################################################
