import sys
from assets import Assets
from locals import *
from score import Score

class Game(object):
    DEBUG_MODE = True

    GLOBAL_ASSETS = Assets(
        IMAGES = {
            "health-powerup" : "health-powerup.png",
            "life-powerup" : "life-powerup.png",
            "dgp-idle-right" : "dgp.png",
            "dgp-idle-left" : "dgp.png?mirror",
            "hud" : "hud.png",
            "health-bar" : "healthbar.png",
            "icon-pistol" : "icon-pistol.png",
            "icon-belt" : "icon-belt.png",
            "icon-club" : "icon-club.png",
            "icon-flame" : "icon-flame.png",
            "icon-glasses" : "icon-glasses.png",
            "icon-mustache" : "icon-mustache.png",
            "icon-bazooka" : "icon-bazooka.png",
            "invisible-wall" : "invisible_wall.png",
            "level-marker" : "levelmarker.png",
            "fog-overlay" : "fog.png",
            "water-overlay" : "water-overlay.png",
            "bazooka-bullet-right" : "Bazooka_Small.png",
            "pistol-bullet-right" : "Bullet_Small.png",
            "mustache-bullet-right" : "DGP_Moustache_Small.png",
            "bazooka-bullet-left" : "Bazooka_Small.png?mirror",
            "pistol-bullet-left" : "Bullet_Small.png?mirror",
            "mustache-bullet-left" : "DGP_Moustache_Small.png?mirror",
            "title_screen" : "title.png",
            "intro_screen" : "intro.png",
            'gameover_screen' : 'gameover.png',
            'ending_screen' : 'ending.png',
            'credits_screen' : 'credits.png'
        },

        ANIMATIONS = {
            "dgp-walk-left" : [
                ("dgp-walk%d-left.png" % i, 0.1, None, Rect(19, 0, 65, 158)) for i in range(1, 9)
            ],

            "dgp-walk-right" : [
                ("dgp-walk%d-left.png?mirror" % i, 0.1, None, Rect(19, 0, 65, 158)) for i in range(1, 9)
            ],

            "dgp-jump-left" : [
                ("dgp-walk2-left.png", 0.1, None, Rect(19, 0, 65, 158))
            ],

            "dgp-jump-right" : [
                ("dgp-walk2-left.png?mirror", 0.1, None, Rect(19, 0, 65, 158))
            ],

            "dgp-club-left" : [
                ("bat_000%d_Layer-Comp-%d.png" % (i, i+1), 0.05, Rect(0, 31, 73, 26) if 3 <= i <= 5 else None, Rect(75, 0, 70, 158)) for i in range(9)
            ],

            "dgp-club-right" : [
                ("bat_000%d_Layer-Comp-%d.png?mirror" % (i, i+1), 0.05, Rect(0, 31, 73, 26) if 3 <= i <= 5 else None, Rect(75, 0, 70, 158)) for i in range(9)
            ],

            "dgp-laser-left" : [
                ("DGP-Lasers%d.png" % i, 0.3 if i==8 else 0.05, Rect(0, 13, 1301, 22) if i==8 else None, Rect(1295, 0, 65, 158)) for i in range(1, 12)
            ],

            "dgp-laser-right" : [
                ("DGP-Lasers%d.png?mirror" % i, 0.3 if i==8 else 0.05, Rect(0, 13, 1301, 22) if i==8 else None, Rect(1295, 0, 65, 158)) for i in range(1, 12)
            ],

            "dgp-belt-left" : [
                ("dgb-belt-%d.png" % i, 0.1 if i in [4,5] else 0.05, Rect(0, 38, 71, 67) if 4<=i<=5 else None, Rect(71, 0, 71, 158)) for i in range(1, 9)
            ],

            "dgp-belt-right" : [
                ("dgb-belt-%d.png?mirror" % i, 0.1 if i in [4,5] else 0.05, Rect(0, 38, 71, 67) if 4<=i<=5 else None, Rect(71, 0, 71, 158)) for i in range(1, 9)
            ],

            "dgp-flamethrower-right" : [
                ("DGP-flame-%d.png" % i, 0.2, Rect(125, 52, 134, 61), Rect(40, 0, 61, 158)) for i in range(1, 4)
            ],

            "dgp-flamethrower-left" : [
                ("DGP-flame-%d.png?mirror" % i, 0.2, Rect(125, 52, 134, 61), Rect(40, 0, 61, 158)) for i in range(1, 4)
            ],

            "dgp-kick-left" : [
                ("kick%d.png" % i, 0.05, Rect(0, 105, 41, 53), Rect(42, 0, 54, 158)) for i in range(1, 5)
            ],

            "dgp-kick-right" : [
                ("kick%d.png?mirror" % i, 0.05, Rect(0, 105, 41, 53), Rect(42, 0, 54, 158)) for i in range(1, 5)
            ],

            "dgp-bazooka-right" : [
                ("DGP-bazooka.png", 0.5, None, Rect(42, 0, 65, 158))
            ],

            "dgp-bazooka-left" : [
                ("DGP-bazooka.png?mirror", 0.5, None, Rect(42, 0, 65, 158))
            ],

            "dgp-mustache-left" : [
                ("dgp-moustach%d.png?mirror" % i, 0.05, None, Rect(43, 0, 59, 158)) for i in range(1,10)
            ],

            "dgp-mustache-right" : [
                ("dgp-moustach%d.png" % i, 0.05, None, Rect(43, 0, 59, 158)) for i in range(1,10)
            ],

            "dgp-pistol-left" : [
                ("DGP-gun.png?mirror", 0.5, None)
            ],

            "dgp-pistol-right" : [
                ("DGP-gun.png", 0.5, None)
            ],

            "dgp-punch-left" : [
                ("dgp-punch-%d.png" % i, 0.05, Rect(11, 50, 73, 35) if 4 <= i <= 9 else None, Rect(77, 0, 68, 158)) for i in range(1,11)
            ],

            "dgp-punch-right" : [
                ("dgp-punch-%d.png?mirror" % i, 0.05, Rect(11, 50, 73, 35) if 4 <= i <= 9 else None, Rect(77, 0, 68, 158)) for i in range(1,11)
            ],

            "green-walk-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-1.png" % i), 0.1, None, Rect(17, 17, 60, 141)) for i in range(10)
            ],

            "green-walk-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-1.png?mirror" % i), 0.1, None, Rect(17, 17, 60, 141)) for i in range(10)
            ],

            "green-antibazooka-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-2.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antibazooka-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-2.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antipistol-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-3.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antipistol-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-3.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antibelt-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-4.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antibelt-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-4.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antipunch-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-5.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antipunch-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-5.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antistomp-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-6.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antistomp-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-6.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antimustache-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-7.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antimustache-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-7.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antilaser-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-8.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antilaser-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-8.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antiflamethrower-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-9.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antiflamethrower-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-9.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-antikick-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-10.png" % i), 0.1, None) for i in range(10)
            ],

            "green-antikick-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-10.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "green-anticlub-right" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-11.png" % i), 0.1, None) for i in range(10)
            ],

            "green-anticlub-left" : [
                (os.path.join("green-walk", "%d" % i, "green-right%d-11.png?mirror" % i), 0.1, None) for i in range(10)
            ],

            "red-walk-right" : [
                ("red-frame%d.png" % i, 0.1, None, Rect(34, 45, 77, 171)) for i in range(1, 5)
            ],

            "red-walk-left" : [
                ("red-frame%d.png?mirror" % i, 0.1, None, Rect(46, 45, 77, 171)) for i in range(1, 5)
            ],

            "red-antistomp-right" : [
                ("red-frame%d-spike.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antistomp-left" : [
                ("red-frame%d-spike.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antikick-right" : [
                ("red-frame%d-kneepads.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antikick-left" : [
                ("red-frame%d-kneepads.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antibelt-right" : [
                ("red-frame%d-belt.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antibelt-left" : [
                ("red-frame%d-belt.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-anticlub-right" : [
                ("red-frame%d-shield.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-anticlub-left" : [
                ("red-frame%d-shield.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antipistol-right" : [
                ("red-frame%d-vest.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antipistol-left" : [
                ("red-frame%d-vest.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antiflamethrower-right" : [
                ("red-frame%d-fire.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antiflamethrower-left" : [
                ("red-frame%d-fire.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antibazooka-right" : [
                ("red-frame%d-skin.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antibazooka-left" : [
                ("red-frame%d-skin.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antilaser-right" : [
                ("red-frame%d-glasses.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antilaser-left" : [
                ("red-frame%d-glasses.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antimustache-right" : [
                ("red-frame%d-mustache.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antimustache-left" : [
                ("red-frame%d-mustache.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antipunch-right" : [
                ("red-frame%d-helmet.png" % i, 0.1, None) for i in range(1, 5)
            ],

            "red-antipunch-left" : [
                ("red-frame%d-helmet.png?mirror" % i, 0.1, None) for i in range(1, 5)
            ]
        },

        FONTS = {
            "lives-font" : ("Orbitron_Medium.ttf", 48)
        },

        SFX = {
             'bazooka1': os.path.join('voices', 'bazooka1.ogg'),
             'bazooka2': os.path.join('voices', 'bazooka2.ogg'),
             'bazooka3': os.path.join('voices', 'bazooka3.ogg'),
             'boss_battle': os.path.join('voices', 'boss_battle.ogg'),
             'club1': os.path.join('voices', 'club1.ogg'),
             'club2': os.path.join('voices', 'club2.ogg'),
             'club3': os.path.join('voices', 'club3.ogg'),
             'end_sequence1': os.path.join('voices', 'end_sequence1.ogg'),
             'end_sequence2': os.path.join('voices', 'end_sequence2.ogg'),
             'end_sequence2-censored': os.path.join('voices', 'end_sequence2-censored.ogg'),
             'flamethrower1': os.path.join('voices', 'flamethrower1.ogg'),
             'flamethrower2': os.path.join('voices', 'flamethrower2.ogg'),
             'flamethrower3': os.path.join('voices', 'flamethrower3.ogg'),
             'kick1': os.path.join('voices', 'kick1.ogg'),
             'kick2': os.path.join('voices', 'kick2.ogg'),
             'kick3': os.path.join('voices', 'kick3.ogg'),
             'laser1': os.path.join('voices', 'laser1.ogg'),
             'laser2': os.path.join('voices', 'laser2.ogg'),
             'laser3': os.path.join('voices', 'laser3.ogg'),
             'laser4': os.path.join('voices', 'laser4.ogg'),
             'moustache1': os.path.join('voices', 'moustache1.ogg'),
             'moustache2': os.path.join('voices', 'moustache2.ogg'),
             'moustache3': os.path.join('voices', 'moustache3.ogg'),
             'pistol1': os.path.join('voices', 'pistol1.ogg'),
             'pistol2': os.path.join('voices', 'pistol2.ogg'),
             'pistol3': os.path.join('voices', 'pistol3.ogg'),
             'punch1': os.path.join('voices', 'punch1.ogg'),
             'punch2': os.path.join('voices', 'punch2.ogg'),
             'punch3': os.path.join('voices', 'punch3.ogg'),
             'stomp1': os.path.join('voices', 'stomp1.ogg'),
             'stomp2': os.path.join('voices', 'stomp2.ogg'),
             'stomp3': os.path.join('voices', 'stomp3.ogg'),
             'whip1': os.path.join('voices', 'whip1.ogg'),
             'whip2': os.path.join('voices', 'whip2.ogg'),
             'whip3': os.path.join('voices', 'whip3.ogg'),
             'whip5': os.path.join('voices', 'whip5.ogg'),

             '1up' : (os.path.join('sfx', '1up-sitar.ogg'), 0.7),
             'bazooka-hit' : (os.path.join('sfx', 'bazooka-hit.ogg'), 0.4),
             'bazooka-shot' : (os.path.join('sfx', 'bazooka-shot.ogg'), 0.9),
             'car-chirp' : (os.path.join('sfx', 'car-chirp.ogg'), 0.7),
             'club' : (os.path.join('sfx', 'club.ogg'), 0.7),
             'flamethrower' : (os.path.join('sfx', 'flamethrower.ogg'), 0.1),
             'Gnorlm-green' : (os.path.join('sfx', 'Gnorlm-green.ogg'), 0.8),
             'gun' : os.path.join('sfx', 'gun.ogg'),
             'Knarxal-brown' : (os.path.join('sfx', 'Knarxal-brown.ogg'), 0.4),
             'lazer' : (os.path.join('sfx', 'lazer.ogg'), 0.5),
             'mustach__woosh' : (os.path.join('sfx', 'mustach__woosh.ogg'), 0.5),
             'punchkick' : (os.path.join('sfx', 'punchkick.ogg'), 0.1),
             'shotgun-cocking' : (os.path.join('sfx', 'shotgun-cocking.ogg'), 0.7),
             'WhipCrack' : (os.path.join('sfx', 'WhipCrack.ogg'), 0.4),

             'intro_music' : ('intro.ogg', 0.1),
             'intro_voice' : os.path.join('voices', 'intro_sequence.ogg'),
             'gameover_sound' : ('gameover.ogg', 0.5),
             'intermission' : (os.path.join('music', 'intermission.ogg'), 0.1),
             'outerspaceboss' : (os.path.join('music', 'outerspaceboss.ogg'), 0.5)
        },

        LEVELS = {
            "level1" : "level1.json",
            "level2" : "level2.json",
            "level3" : "level3.json",
            "level4" : "level4.json",
            "level6" : "level6.json",
            "level7" : "level7.json",
            "level8" : "level8.json"
        }
    )

    PER_LEVEL_ASSETS = {
        "level1" : Assets(
            IMAGES = {
                "floor" : "floor.png",
                "platform" : "platform.png",
                "block" : "barrier.png",
                "pit-left" : "pit-left.png",
                "pit-right" : "pit-right.png"
            },

            BACKGROUNDS = {
                "background" : ("bg01.jpg", 'horizontal')
            },

            SFX = {
                "background-music" : (os.path.join("music", "outerspace.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level1.ogg")
            }
        ),

        "level2" : Assets(
            IMAGES = {
                "floor" : "floor2.png",
                "floor-top" : "floor2-top.png",
                "platform" : "platform2.png",
                "weak-platform" : "platform2-weak.png",
                "block" : "barrier2.png",
                "block-top" : "barrier2-top.png",
                "pit-left" : "pit-left2.png",
                "pit-right" : "pit-right2.png"
            },

            BACKGROUNDS = {
                "background" : ("bg02.jpg", 'horizontal')
            },

            SFX = {
                "background-music" : (os.path.join("music", "junkyard.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level2.ogg")
            }
        ),

        "level3" : Assets(
            IMAGES = {
                "platform" : "platform3.png",
                "weak-platform" : "platform3-weak.png",
            },

            BACKGROUNDS = {
                "background" : ("bg03.jpg", 'vertical')
            },

            SFX = {
                "background-music" : (os.path.join("music", "waterfalls.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level3.ogg")
            }
        ),

        "level4" : Assets(
            IMAGES = {
                "platform" : "platform4a.png",
                "platform2" : "platform4b.png",
                "floor" : "rock4.png",
            },

            BACKGROUNDS = {
                "background" : ("bg04.jpg", 'super_vertical')
            },

            SFX = {
                "background-music" : (os.path.join("music", "downhill.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level4.ogg")
            }
        ),

        "level5" : Assets(
            SFX = {
                "space-lion-theme" : (os.path.join("music", "spaceliontheme.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level5.ogg")
            }
        ),

        "level6" : Assets(
            IMAGES = {
                "floor" : "floor6.png",
                "platform" : "platform6.png",
                "weak-platform" : "platform6-weak.png",
                "block" : "barrier6.png",
                "stalagmite" : "stalagmite-100.png",
                "stalactite1" : "stalactite-100.png",
                "stalactite2" : "stalactite-200.png"
            },

            BACKGROUNDS = {
                "background" : ("bg06.jpg", 'horizontal')
            },

            SFX = {
                "background-music" : (os.path.join("music", "cave.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level6.ogg"),
                'dripping_water' : (os.path.join('sfx', 'dripping_water.ogg'), 0.3),
            }
        ),

        "level7" : Assets(
            IMAGES = {
                "floor" : "floor7.png",
                "platform" : "platform7.png",
                "weak-platform" : "platform7-weak.png",
                "block" : "barrier7.png",
                "pit-left" : "pit-left7.png",
                "pit-right" : "pit-right7.png",
            },

            BACKGROUNDS = {
                "background" : ("bg07.jpg", 'horizontal')
            },

            SFX = {
                "background-music" : (os.path.join("music", "underwater.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level7.ogg"),
                'bubbles' : (os.path.join('sfx', 'bubbles.ogg'), 0.35),
                'in_the_water' : (os.path.join('sfx', 'in_the_water.ogg'), 0.55)
            }
        ),

        "level8" : Assets(
            IMAGES = {
                "platform" : "platform8.png",
                "weak-platform" : "platform8-weak.png",
                "stalagmite" : "stalagmite-500.png",
                "stalactite1" : "stalactite-500.png",
                "stalactite2" : "stalactite-500.png"
            },

            BACKGROUNDS = {
                "background" : ("bg08.jpg", 'horizontal')
            },

            SFX = {
                "background-music" : (os.path.join("music", "lava.ogg"), 0.5),
                "level-voice" : os.path.join("voices", "level8.ogg"),
                'lava' : (os.path.join('sfx', 'lava.ogg'), 0.4),
            }
        ),
    }

    def __init__(self):
        pygame.init()

        if len(sys.argv) >= 2 and sys.argv[1] in ['-w', '--windowed']:
            self.display_flags = 0
        else:
            self.display_flags = pygame.HWSURFACE | pygame.FULLSCREEN

        self.init_screen()
        self.show_loading()
        self.load_assets()

        self.ticks = 0.0
        self.clock = pygame.time.Clock()

        self.reload()

    def get_ticks(self):
        return self.ticks

    def tick(self):
        self.ticks += MSECS_PER_FRAME
        #fps = max(self.clock.get_fps(), 30)
        #self.ticks += 1000.0 / fps
        self.clock.tick(FPS)

    def get_fps(self):
        return self.clock.get_fps()

    def reload(self):
        self.score = Score()
        self.cheating = False
        self.cheat_idx = 0

        self.music_fn = ""
        self.music_channel = None

    def show_loading(self):
        loading = pygame.font.Font(os.path.join("data", "Orbitron_Medium.ttf"), 48).render("Loading...", False, (255, 255, 255))
        rect = loading.get_rect()
        rect.centerx = SCREEN_WIDTH / 2
        rect.centery = SCREEN_HEIGHT / 2
        self.screen.fill((0, 0, 0))
        self.screen.blit(loading, (rect.left, rect.top))

        pygame.display.update()

    def load_assets(self):
        self.global_assets = self.GLOBAL_ASSETS.load_assets()
        #self.per_level_assets = { name : assets.load_assets() for name, assets in self.PER_LEVEL_ASSETS.items() }
        self.assets = self.global_assets

    def choose_assets(self, level_name):
        if self.PER_LEVEL_ASSETS.has_key(level_name):
            self.show_loading()
            this_level_assets = self.PER_LEVEL_ASSETS[level_name].load_assets()
        else:
            this_level_assets = {}

        self.assets = {}
        self.assets.update(self.global_assets)
        self.assets.update(this_level_assets)

    def set_level(self, level_class):
        self.choose_assets(level_class.LEVEL_ASSET)
        self.level = level_class()
        self.update_background_music()

    def init_screen(self):
        # pygame.display.set_icon(pygame.image.load(self._fullfn("icon.png")))
        pygame.display.set_caption(SCREEN_CAPTION)
        self.screen = pygame.display.set_mode(SCREEN_SIZE, self.display_flags, 32)
        pygame.mouse.set_visible(False) # We have no need for a cursor

    def toggle_fullscreen(self):
        self.display_flags ^= pygame.FULLSCREEN
        self.init_screen()

    def handle_event(self, event):
        # return True means screen should be refreshed

        if event.type == KEYDOWN and event.key == K_f:
            self.toggle_fullscreen()
            return True

        elif event.type == KEYDOWN and event.key == K_l:
            if self.cheat_idx == 0:
                self.cheat_idx = 1
        elif event.type == KEYDOWN and event.key == K_i:
            if self.cheat_idx == 1:
                self.cheat_idx = 2
        elif event.type == KEYDOWN and event.key == K_o:
            if self.cheat_idx == 2:
                self.cheat_idx = 3
        elif event.type == KEYDOWN and event.key == K_n:
            if self.cheat_idx == 3:
                self.cheat_idx = 0
                g.cheat()

        return False

    def cheat(self):
        self.cheating = True
        self.score.lives = 99
        self.score.health = MAX_HEALTH
        self.score.all_weapons_cheat()

    def update_background_music(self):
        cur_music_obj = self.assets.get("background-music", {'fn':''})

        if self.music_fn == cur_music_obj['fn']:
            return # already playing

        self.music_fn = cur_music_obj['fn']

        self.stop_music()

        if cur_music_obj.has_key('sound'):
            # start the new music
            snd = cur_music_obj['sound']
            self.music_channel = snd.play(loops=-1)

    def stop_music(self):
        if self.music_channel is not None:
            # stop the current music
            self.music_channel.stop()
            self.music_channel = None

    def play(self):
        from levels import LevelSet
        from intro_screen import TitleScreen, IntroScreen, EndingScreen, CreditsScreen, GameOverScreen

        while True:
            self.reload()
            self.ls = LevelSet()

            if TitleScreen(self.screen).play() != LEVEL_BEATEN:
                return

            if IntroScreen(self.screen).play() != LEVEL_BEATEN:
                return

            while self.score.lives > 0 and self.ls.level is not None:
                self.set_level(self.ls.level)
                self.level.play()

                if self.level.result == USER_EXIT:
                    return

                elif self.level.result == PLAYER_DEAD:
                    self.score.lives -= 1

                elif self.level.result == LEVEL_BEATEN:
                    self.ls.next_level()

            self.stop_music()

            if self.ls.level is None:
                # won game
                break
            else:
                # game over
                if GameOverScreen(self.screen).play() != LEVEL_BEATEN:
                    return

        EndingScreen(self.screen).play()
        CreditsScreen(self.screen).play()


################################################################################

g = Game()
