import os
import sys
import pygame
import json
from pprint import pprint
from glob import glob

####################################################################################################

SCREEN_HEIGHT = 768

COLOR_TABLE = {
    (0, 0, 0) : "floor",
    (127, 0, 0) : "pit_left",
    (255, 0, 0) : "pit_right",
    (0, 127, 0) : "platform",
    (0, 227, 0) : "platform2",
    (127, 127, 0) : "weak_platform",
    (255, 127, 0) : "block",

    (0, 255, 0) : "stalactite1", # pixel indicates bottom left corner
    (127, 255, 0) : "stalactite2", # pixel indicates bottom left corner
    (255, 255, 0) : "stalagmite",

    (0, 0, 127) : "weapon",

    (0, 127, 127) : "pickup1",
    (127, 127, 127) : "pickup2",
    (255, 127, 127) : "pickup3",
    (0, 255, 127) : "pickup4",
    (127, 255, 127) : "pickup5",
    (255, 255, 127) : "pickup6",

    (0, 127, 255) : "player",
    (127, 127, 255) : "enemy1",
    (255, 127, 255) : "enemy2",

    (192, 168, 0) : 'end_sign'
}

class LevelCompiler(object):
    def __init__(self, in_fn, out_fn):
        self.img = pygame.image.load(in_fn)
        rect = self.img.get_rect()
        self.width, self.height = rect.width, rect.height

        #self.factor = float(SCREEN_HEIGHT) / self.height
        self.factor = 10
        self.out_fn = out_fn

    def at(self, x, y):
        c = self.img.get_at((x, y))
        return c.r, c.g, c.b

    def _to_real_coordinates(self, x, y):
        return int(x * self.factor), int(y * self.factor)

    def process(self):
        print "Width, Height:", self.width, "x", self.height
        self.dimensions = self._to_real_coordinates(self.width, self.height)

        self.stuff = {}

        for y in range(self.height):
            for x in range(self.width):
                c = self.at(x, y)

                if COLOR_TABLE.has_key(c):
                    self.stuff.setdefault(COLOR_TABLE[c], []).append(self._to_real_coordinates(x, y))

        pprint(self.stuff)

    def save(self):
        json.dump({
            "dimensions" : self.dimensions,
            "stuff" : self.stuff
        }, file(self.out_fn, 'wb'))

    def compile(self):
        self.process()
        self.save()

####################################################################################################

def compile_all():
    for in_fn in glob(r"levels\*.png"):
        out_fn = os.path.join("data", os.path.splitext(os.path.basename(in_fn))[0] + '.json')
        print "+++++++++"
        print "+ Compiling %s -> %s" % (in_fn, out_fn)
        print "+++++++++++++++++++++++++++++++++++"
        LevelCompiler(in_fn, out_fn).compile()

####################################################################################################

def usage():
    print "lc - Level Compiler"
    print
    print "Usage1: lc.py [input.png] [output.json]"
    print "Compiles one level."
    print
    print "Usage2: lc.py all"
    print 'Compiles all levels from the "levels" directory to the "data" directory.'

####################################################################################################

def main():
    if len(sys.argv) == 3:
        in_fn, out_fn = sys.argv[1], sys.argv[2]
        LevelCompiler(in_fn, out_fn).compile()
    elif len(sys.argv) == 2 and sys.argv[1] == 'all':
        compile_all()
    else:
        usage()

####################################################################################################

if __name__ == '__main__':
    main()
