from locals import *
from level import Level
from ddr_level import DDRLevel
from powerup import PistolPickup, MustachePickup, BazookaPickup, LasersPickup, ClubPickup, BeltPickup
import random
from game import g

class Level1(Level):
    LEVEL_ASSET = "level1"
    LEVEL_WEAPON = BeltPickup

class Level2(Level):
    LEVEL_ASSET = "level2"
    LEVEL_WEAPON = ClubPickup

class Level3(Level):
    LEVEL_ASSET = "level3"
    LEVEL_WEAPON = PistolPickup
    HORIZONTAL = False

class Level4(Level):
    LEVEL_ASSET = "level4"
    LEVEL_WEAPON = BeltPickup # does not really have a weapon...
    HORIZONTAL = False
    SUPERSCROLLING_SPEED = 100
    LOW_GRAVITY = True

    ENEMY_SPAWN_RANGE = (400, 1200)
    ENEMY_SPAWN_RATE = 2500 # msecs
    NUM_TO_KILL = 6

    def __init__(self):
        Level.__init__(self)

        self.t = None
        self.kill_counter = 0

    def tick(self):
        if self.t is None:
            self.t = g.get_ticks()
            return

        t = g.get_ticks() - self.t

        if t >= self.ENEMY_SPAWN_RATE:
            self.spawn_enemy(int(random.uniform(*self.ENEMY_SPAWN_RANGE)), -50)
            d, m = divmod(t, self.ENEMY_SPAWN_RATE)
            self.t += d * self.ENEMY_SPAWN_RATE

    def kill_enemy(self, enemy, attack):
        Level.kill_enemy(self, enemy, attack)
        self.kill_counter += 1

        if self.kill_counter == self.NUM_TO_KILL:
            self.result = LEVEL_BEATEN
            self.looping = False

# level 5 gives you a Flamethrower

class Level6(Level):
    LEVEL_ASSET = "level6"
    LEVEL_WEAPON = BazookaPickup
    DARK = "fog-overlay"

    def play(self):
        chan = g.assets['dripping_water']['sound'].play(loops=-1)
        Level.play(self)
        chan.stop()

class Level7(Level):
    LEVEL_ASSET = "level7"
    LEVEL_WEAPON = LasersPickup
    LOW_GRAVITY = True
    DARK = "water-overlay"

    def play(self):
        chan1 = g.assets['in_the_water']['sound'].play()
        chan2 = g.assets['bubbles']['sound'].play(loops=-1)
        Level.play(self)
        chan1.stop()
        chan2.stop()

class Level8(Level):
    LEVEL_ASSET = "level8"
    LEVEL_WEAPON = MustachePickup

    def play(self):
        chan = g.assets['lava']['sound'].play(loops=-1)
        Level.play(self)
        chan.stop()

class LevelSet(object):
    _LEVELS = [Level1, Level2, Level3, Level4, DDRLevel, Level6, Level7, Level8]

    def __init__(self):
        self._idx = 0

    @property
    def level(self):
        if self._idx < len(self._LEVELS):
            return self._LEVELS[self._idx]
        else:
            return None

    def next_level(self):
        self._idx += 1

    def cheat(self, idx):
        self._idx = min(idx, len(self._LEVELS) - 1)
