from locals import *
from game_object import GameObject
from game import g

### Types ########################################

class Platform(GameObject):
    is_solid = False
    walk_through = True

    def step(self):
        pass

class Decor(Platform):
    pass

class SolidPlatform(Platform):
    is_solid = True

class WeakPlatform(SolidPlatform):
    TIME_TO_DESOLIDIFY = 500
    TIME_TO_REAPPEAR = 5000 # from start

    DISAPPEAR_DURATION = 1000
    PIXELS_TO_FALL = 800

    def __init__(self, *args):
        SolidPlatform.__init__(self, *args)
        self.orig_disp_rect = self.disp_rect

        self.reset()

    def reset(self):
        self.triggered = False
        self.de_solidified = False
        self.disappeared = False
        self.disp_rect = self.orig_disp_rect

    def step(self):
        if self.triggered:
            return

        self.triggered = True
        self.trigger_time = g.get_ticks()

    def de_solidify(self):
        self.start_disappear = g.get_ticks()

        self.de_solidified = True

    def totally_disappear(self):
        self.disappeared = True
        self.disp_rect.right = -100 # there is never anything over there :)

    def disappear_update(self):
        if self.disappeared:
            return

        t = float(g.get_ticks() - self.start_disappear) / self.DISAPPEAR_DURATION
        if t >= 1:
            self.totally_disappear()
            return

        pixels_down = int(self.PIXELS_TO_FALL * t)

        self.disp_rect = self.orig_disp_rect.move(0, pixels_down)

    def update(self, *args):
        SolidPlatform.update(self, *args)

        if self.triggered:
            t = g.get_ticks() - self.trigger_time
            if t > self.TIME_TO_DESOLIDIFY and not self.de_solidified:
                self.de_solidify()

            if t > self.TIME_TO_DESOLIDIFY:
                self.disappear_update()

            if t > self.TIME_TO_REAPPEAR:
                self.reset()

class Wall(SolidPlatform):
    walk_through = False

### Specific platforms ###########################

class Floor(Wall):
    ASSET_NAME = "floor"

class FloorTop(Decor):
    ASSET_NAME = "floor-top"
    POSITION_BY = POSITION_BY_BOTTOM_LEFT

class InvisibleWall(Wall):
    ASSET_NAME = "invisible-wall"
    POSITION_BY = POSITION_BY_TOP_RIGHT

    def update(self, *args):
        Wall.update(self, *args)

        self.disp_rect.centery = g.level.player.char_rect.centery

        # ugliest hack ever - 2am last day, sorry
        # (allow winning level 3)
        if not g.level.HORIZONTAL and g.level.height > 4000:
            if self.disp_rect.top < 500:
                self.disp_rect.top = 500

class InvisibleWallRight(InvisibleWall):
    POSITION_BY = POSITION_BY_TOP_LEFT

class Block(Wall):
    ASSET_NAME = "block"

class BlockTop(Decor):
    ASSET_NAME = "block-top"
    POSITION_BY = POSITION_BY_BOTTOM_LEFT

class RegularPlatform(SolidPlatform):
    ASSET_NAME = "platform"

class AlternatePlatform(SolidPlatform):
    ASSET_NAME = "platform2"

class RegularWeakPlatform(WeakPlatform):
    ASSET_NAME = "weak-platform"

class PitLeft(Decor):
    ASSET_NAME = "pit-left"

class PitRight(Decor):
    ASSET_NAME = "pit-right"

class EndSign(Decor):
    ASSET_NAME = "level-marker"
    POSITION_BY = POSITION_BY_BOTTOM_RIGHT

class DarknessOverlay(Decor):
    def update(self, *args):
        Decor.update(self, *args)

        self.disp_rect.center = g.level.player.char_rect.center

class FogOverlay(DarknessOverlay):
    ASSET_NAME = "fog-overlay"

class WaterOverlay(DarknessOverlay):
    ASSET_NAME = "water-overlay"

class Stalactite1(Decor):
    ASSET_NAME = "stalactite1"
    POSITION_BY = POSITION_BY_BOTTOM_LEFT

class Stalactite2(Decor):
    ASSET_NAME = "stalactite2"
    POSITION_BY = POSITION_BY_BOTTOM_LEFT

class Stalagmite(Decor):
    ASSET_NAME = "stalagmite"
    POSITION_BY = POSITION_BY_TOP_LEFT
