from bullet import BulletBase
from locals import *
from character import Character
from melee import Punch, Kick, Melee
from game import g
import random

class Player(Character):
    ANIMATION = { DIR_LEFT : "dgp-walk-left", DIR_RIGHT : "dgp-walk-right" }
    ANIMATION_IDLE = { DIR_LEFT : "dgp-idle-left", DIR_RIGHT : "dgp-idle-right" }
    ANIMATION_JUMP = { DIR_LEFT : "dgp-jump-left", DIR_RIGHT : "dgp-jump-right" }

    INITIAL_WALK_VELOC = 200   # pixels per second
    WALK_ACCEL = 1000          # pixels per second^2
    WALK_ACCEL_DURATION = 0.2  # seconds of acceleration

    @property
    def initial_jump_veloc_stomp(self):
        return self.initial_jump_veloc / 2

    def stomp_jump(self):
        self.start_falling()
        self.vert_veloc = -self.initial_jump_veloc_stomp

    def __init__(self, initial_x, initial_y):
        Character.__init__(self, initial_x, initial_y, DIR_RIGHT)

        self.melee = None
        self.triggers_weak_platforms = True

    @property
    def in_melee(self):
        return self.melee is not None

    def do_melee(self, melee_class):
        # No punching in free-fall or while already punching
        if self.falling or self.in_melee:
            return

        self.sound_weapon(melee_class)

        # Stop moving
        self.stop_walk()
        self.move_allowed = False

        # Create a melee object
        self.melee = melee_class(self.dir_facing)
        self.animation = self.melee.animation
        self.pos = self.pos # update display position due to change of animation

    def resume_after_melee_or_shoot(self):
        self.melee = None
        self.move_allowed = True

        # Resume walking
        if self.left_pressed or self.right_pressed:
            self.start_walk()
        else:
            # At least turn off that pesky animation
            self.update_anim()

    def update_anim(self):
        Character.update_anim(self)

        if not self.move_allowed:
            self.resume_after_melee_or_shoot() # make sure...

    def do_bullet(self, bullet_class):
        # No shooting in free-fall
        if self.falling:
            return

        self.sound_weapon(bullet_class)

        # Stop moving
        self.move_allowed = False
        self.stop_walk()

        # Create a bullet
        bullet = bullet_class()
        g.level.bullets.add(bullet)
        self.animation = bullet.shoot_animation
        self.pos = self.pos # update display position due to change of animation

    @property
    def hit_rect(self):
        hr = self.animation.hit_rect

        if hr is None:
            return None

        return hr.move(*self.disp_rect.topleft)

    def handle_event(self, event):
        if event.type == KEYDOWN and event.key == K_LEFT:
            self.start_walk_left()

        elif event.type == KEYDOWN and event.key == K_RIGHT:
            self.start_walk_right()

        elif event.type == KEYUP and event.key == K_LEFT:
            self.stop_walk_left()

        elif event.type == KEYUP and event.key == K_RIGHT:
            self.stop_walk_right()

        elif event.type == KEYDOWN and event.key == K_UP:
            self.jump()

        elif event.type == KEYDOWN and event.key == K_a:
            self.do_melee(Punch)

        elif event.type == KEYDOWN and event.key == K_s:
            self.do_melee(Kick)

        elif event.type == KEYDOWN and event.key == K_d:
            self.do_weapon()

        elif event.type == KEYDOWN and event.key == K_q:
            g.score.prev_weapon()

        elif event.type == KEYDOWN and event.key == K_e:
            g.score.next_weapon()

    def hurt(self):
        # Player's hurt! This returns True if we're still alive, False if we died.

        # First, are we in mercy invincibility?
        if self.blinking:
            return True

        # Okay, so we gotta take a hit. Do we have health to spare?
        if g.score.health == 1:
            return False # :(

        # Take a hit and lose a health point
        self.start_blinking()
        g.score.health -= 1
        return True

    def do_weapon(self):
        weapon = g.score.weapon

        if weapon is None:
            return
        elif issubclass(weapon, Melee):
            self.do_melee(weapon)
        elif issubclass(weapon, BulletBase):
            self.do_bullet(weapon)

    def sound_weapon(self, weapon_class):
        if weapon_class.USE_SFX is not None:
            g.assets[weapon_class.USE_SFX]['sound'].play()

        if not weapon_class.WEAPON_SOUNDS:
            return

        if random.randrange(5):
            return

        sound = random.choice(weapon_class.WEAPON_SOUNDS)
        if g.assets.has_key(sound) and g.assets[sound].has_key('sound'):
            g.assets[sound]['sound'].play()
